@@ -47,6 +47,7 @@ import re
 import sys
 
 import httplib2
+import jsonschema
 from keystoneclient.v2_0 import client as keystoneclient
 
 import solumclient
@@ -564,11 +565,22 @@ Available commands:
 
         Register a new application with Solum.
 
-    solum app deploy <NAME|ID>
+    solum app deploy <NAME|ID> --du-id <du-id>
         Deploy an application, building any applicable artifacts first.
+        du-id is optional flag. It can be used to pass in id of a previously
+        created deployment unit. If passed, this command will deploy that
+        du instead of building one first.
 
     solum app delete <NAME|ID>
         Delete an application and all related artifacts.
+
+    solum app scale <NAME|ID> <scaling target>
+
+    solum app logs <NAME|UUID> [--wf-id <wf-id>]
+        Show the logs of an application for all the workflows.
+        wf-id is optional flag which can be used to pass in id of one of
+        the existing workflows. If provided, the logs only for that workflow
+        are displayed.
     """
     def _validate_app_file(self, app_data):
         if ('workflow_config' in app_data and
@@ -613,15 +625,17 @@ Available commands:
         return app_name
 
     def _get_and_validate_languagepack(self, app_data, args):
-        # Check for the language pack. Check args first, then planfile.
+        # Check for the language pack. Check args first, then appfile.
         # If it's neither of those places, prompt for it and update the
-        # plan definition.
+        # app data.
 
         languagepack = None
         if args.languagepack is not None:
             languagepack = args.languagepack
-            # check if given languagepack exists or not
-            lp = None
+        elif app_data.get('languagepack') is not None:
+            languagepack = app_data.get('languagepack')
+        # check if given languagepack exists or not
+        if languagepack:
             try:
                 lp = self.client.languagepacks.find(name_or_id=languagepack)
             except Exception:
@@ -631,7 +645,7 @@ Available commands:
                 raise exc.CommandError("No languagepack in READY state. "
                                        "Create a languagepack first.")
             app_data['languagepack'] = languagepack
-        elif app_data.get('languagepack') is None:
+        else:
             languagepacks = self.client.languagepacks.list()
             filtered_list = cli_utils.filter_ready_lps(languagepacks)
 
@@ -727,6 +741,7 @@ Available commands:
             app_data['ports'] = port_list
 
     def _get_parameters(self, app_data, args):
+        app_data['parameters'] = {}
         if args.param_file is not None:
             try:
                 with open(args.param_file) as param_f:
@@ -822,16 +837,47 @@ Available commands:
                 'description': 'default app description.',
                 'source': {
                     'repository': '',
-                    'revision': 'master'
+                    'revision': 'master',
+                    'private': False,
+                    'repo_token': ''
                 },
                 'workflow_config': {
                     'test_cmd': '',
                     'run_cmd': ''
                 },
                 'trigger_actions': ['build', 'deploy'],
-                'repo_token': ''
             }
 
+        app_name = self._get_and_validate_app_name(app_data, args)
+        app_data['name'] = app_name
+
+        self._get_and_validate_languagepack(app_data, args)
+
+        self._get_app_repo_details(app_data, args)
+
+        self._get_run_command(app_data, args)
+
+        self._get_unittest_command(app_data, args)
+
+        self._get_port(app_data, args)
+
+        self._get_parameters(app_data, args)
+
+        self._validate_app_data(app_data)
+
+        app = self.client.apps.create(**app_data)
+
+        self._setup_github_trigger(app_data, app, args)
+
+        app.trigger = app.trigger_actions
+        app.workflow = app.workflow_config
+
+        fields = ['name', 'id', 'created_at', 'description', 'languagepack',
+                  'ports', 'source', 'workflow',
+                  'trigger_uuid', 'trigger', 'trigger_uri']
+        self._print_dict(app, fields, wrap=72)
+
+    def _validate_app_data(self, app_data):
         # app file schema
         schema = {
             "title": "app file schema",
@@ -850,63 +896,65 @@ Available commands:
                     "type": "string"
                 },
                 "source": {
-                    "type": "object"
-                },
-                "workflow_config": {
-                    "type": "object"
-                },
-                "repo_token": {
-                    "type": "string"
+                    "type": "object",
+                    "properties": {
+                        "repository": {
+                            "type": "string"
+                        },
+                        "revision": {
+                            "type": "string"
+                        },
+                        "private": {
+                            "type": "boolean"
+                        },
+                        "repo_token": {
+                            "type": "string"
+                        }
+                    }
                 },
                 "trigger_actions": {
-                    "type": "array"
+                    "type": "array",
+                    "items": {
+                        "type": "string",
+                        "enum": ["unittest", "build", "deploy"]
+                    }
+                },
+                "workflow_config": {
+                    "type": "object",
+                    "properties": {
+                        "test_cmd": {
+                            "type": "string"
+                        },
+                        "run_cmd": {
+                            "type": "string"
+                        }
+                    }
                 },
                 "ports": {
-                    "type": "array"
+                    "type": "array",
+                    "items": {
+                        "type": "integer"
+                    }
+                },
+                "parameters": {
+                    "type": "object"
                 }
             },
-            "required": ["version", "name", "description",
-                         "languagepack", "source", "workflow_config",
-                         "trigger_actions", "ports"]
+            "required": [
+                "version",
+                "name",
+                "description",
+                "languagepack",
+                "source",
+                "trigger_actions",
+                "workflow_config",
+                "ports"
+            ]
         }
-
-        app_name = self._get_and_validate_app_name(app_data, args)
-        app_data['name'] = app_name
-
-        self._get_and_validate_languagepack(app_data, args)
-
-        self._get_app_repo_details(app_data, args)
-
-        self._get_run_command(app_data, args)
-
-        self._get_unittest_command(app_data, args)
-
-        self._get_port(app_data, args)
-
-        self._get_parameters(app_data, args)
-
-        #  TODO(vijendar): currently doing very basic validation.
-        # Need to implement more robust schema based validation
-        appdata_keys = app_data.keys()
-        appdata_keys.sort()
-        schema_keys = schema['properties'].keys()
-        schema_keys.sort()
-        if appdata_keys != schema_keys:
-            message = "Unknown key(s) in app data file: %s" % list(
-                set(appdata_keys) - set(schema_keys))
-            raise exc.CommandError(message=message)
-
-        app = self.client.apps.create(**app_data)
-
-        self._setup_github_trigger(app_data, app, args)
-
-        app.trigger = app.trigger_actions
-        app.workflow = app.workflow_config
-
-        fields = ['name', 'id', 'created_at', 'description', 'languagepack',
-                  'ports', 'source', 'workflow',
-                  'trigger_uuid', 'trigger', 'trigger_uri']
-        self._print_dict(app, fields, wrap=72)
+        try:
+            jsonschema.validate(app_data, schema)
+        except jsonschema.exceptions.ValidationError as exp:
+            raise exc.CommandError(message=str(exp))
 
     def update(self):
         """Update the registration of an existing app."""
@@ -978,12 +1026,17 @@ Available commands:
         args = self.parser.parse_args()
         app = self.client.apps.find(name_or_id=args.name)
 
-        app.trigger = app.trigger_actions
-        app.workflow = app.workflow_config
-
         fields = ['name', 'id', 'created_at', 'description', 'languagepack',
                   'entry_points', 'ports', 'source',
                   'trigger_uuid', 'trigger', 'app_url']
+
+        app.trigger = app.trigger_actions
+        app.workflow = app.workflow_config
+        if app.scale_config:
+            if app.scale_config[args.name]['target']:
+                app.target_instances = app.scale_config[args.name]['target']
+                fields.append('target_instances')
+
         self._print_dict(app, fields, wrap=72)
 
         wfman = cli_wf.WorkflowManager(self.client, app_id=app.id)
@@ -1000,16 +1053,37 @@ Available commands:
         cli_app.AppManager(self.client).delete(
             app_id=str(app.id))
 
-    def _create_workflow(self, actions):
-        self.parser.add_argument('name')
-        args = self.parser.parse_args()
-        app = self.client.apps.find(name_or_id=args.name)
+    def _create_scaling_workflow(self, actions, app_name_id, tgt):
+        app = self.client.apps.find(name_or_id=app_name_id)
+        wf = (cli_wf.WorkflowManager(self.client,
+                                     app_id=app.id).create(actions=actions,
+                                                           scale_target=tgt))
+        fields = ['wf_id', 'app_id', 'actions', 'config',
+                  'source', 'id', 'created_at', 'updated_at']
+        self._print_dict(wf, fields, wrap=72)
+
+    def _create_workflow(self, actions, app_name_id=''):
+        if not app_name_id:
+            app = self.client.apps.find(name_or_id=app_name_id)
+        else:
+            self.parser.add_argument('name')
+            args = self.parser.parse_args()
+            app = self.client.apps.find(name_or_id=args.name)
         wf = (cli_wf.WorkflowManager(self.client,
                                      app_id=app.id).create(actions=actions))
         fields = ['wf_id', 'app_id', 'actions', 'config',
                   'source', 'id', 'created_at', 'updated_at']
         self._print_dict(wf, fields, wrap=72)
 
+    def _create_workflow_for_prebuilt_du(self, actions, app_name_id, du_id):
+        app = self.client.apps.find(name_or_id=app_name_id)
+        wf = (cli_wf.WorkflowManager(self.client,
+                                     app_id=app.id).create(actions=actions,
+                                                           du_id=du_id))
+        fields = ['wf_id', 'app_id', 'actions', 'config',
+                  'source', 'id', 'created_at', 'updated_at']
+        self._print_dict(wf, fields, wrap=72)
+
     def unittest(self):
         """Create a new workflow for an app."""
         actions = ['unittest']
@@ -1022,8 +1096,111 @@ Available commands:
 
     def deploy(self):
         """Create a new workflow for an app."""
-        actions = ['unittest', 'build', 'deploy']
-        self._create_workflow(actions)
+        self.parser.add_argument("name")
+        self.parser.add_argument('--du-id',
+                                 dest="du_id",
+                                 help="ID of the DU image.")
+        args = self.parser.parse_args()
+        if args.du_id:
+            actions = ['deploy']
+            self._create_workflow_for_prebuilt_du(actions, args.name,
+                                                  args.du_id)
+        else:
+            actions = ['unittest', 'build', 'deploy']
+            self._create_workflow(actions, args.name)
+
+    def scale(self):
+        """Scale the app."""
+        self.parser.add_argument('name')
+        self.parser.add_argument('target')
+        args = self.parser.parse_args()
+
+        target = args.target
+        try:
+            target = int(target)
+        except ValueError:
+            msg = "Must provide integer value for scale target."
+            raise exc.CommandException(message=msg)
+
+        if target <= 0:
+            msg = "Scale target must be greater than zero."
+            raise exc.CommandException(message=msg)
+
+        actions = ['scale']
+        self._create_scaling_workflow(actions, args.name, target)
+
+    def _display_logs_for_all_workflows(self, app):
+        wfman = cli_wf.WorkflowManager(self.client, app_id=app.id)
+        wfs = wfman.list()
+
+        all_logs_list = []
+        fields = ["resource_uuid", "created_at"]
+        for wf in wfs:
+            revision = wf.wf_id
+            loglist = wfman.logs(revision_or_id=revision)
+            for log in loglist:
+                all_logs_list.append(log)
+                strategy_info = json.loads(log.strategy_info)
+                if log.strategy == 'local':
+                    if 'local_storage' not in fields:
+                        fields.append('local_storage')
+                    log.local_storage = log.location
+                elif log.strategy == 'swift':
+                    if 'swift_container' not in fields:
+                        fields.append('swift_container')
+                    if 'swift_path' not in fields:
+                        fields.append('swift_path')
+                    log.swift_container = strategy_info['container']
+                    log.swift_path = log.location
+                else:
+                    if 'location' not in fields:
+                        fields.append('location')
+        self._print_list(all_logs_list, fields)
+
+    def logs(self):
+        """Print a list of all logs belonging to a single app."""
+
+        self.parser.add_argument('name')
+
+        self.parser.add_argument('--wf-id',
+                                 dest='wf_id',
+                                 help="Workflow ID")
+
+        args = self.parser.parse_args()
+        app = self.client.apps.find(name_or_id=args.name)
+
+        if args.wf_id:
+            try:
+                revision = int(args.wf_id, 10)
+            except (ValueError, TypeError):
+                revision = args.wf_id
+            display_logs_for_single_workflow(self, app, revision)
+        else:
+            self._display_logs_for_all_workflows(app)
+
+
+def display_logs_for_single_workflow(ref, app, revision):
+    wfman = cli_wf.WorkflowManager(ref.client, app_id=app.id)
+    loglist = wfman.logs(revision_or_id=revision)
+    fields = ["resource_uuid"]
+    for log in loglist:
+        strategy_info = json.loads(log.strategy_info)
+        if log.strategy == 'local':
+            if 'local_storage' not in fields:
+                fields.append('local_storage')
+            log.local_storage = log.location
+        elif log.strategy == 'swift':
+            if 'swift_container' not in fields:
+                fields.append('swift_container')
+            if 'swift_path' not in fields:
+                fields.append('swift_path')
+            log.swift_container = strategy_info['container']
+            log.swift_path = log.location
+        else:
+            if 'location' not in fields:
+                fields.append('location')
+
+    ref._print_list(loglist, fields)
 
 
 class WorkflowCommands(cli_utils.CommandsBase):
@@ -1084,28 +1261,7 @@ Available commands:
         except ValueError:
             revision = args.workflow
         app = self.client.apps.find(name_or_id=args.app)
-
-        wfman = cli_wf.WorkflowManager(self.client, app_id=app.id)
-        loglist = wfman.logs(revision_or_id=revision)
-        fields = ["resource_uuid"]
-        for log in loglist:
-            strategy_info = json.loads(log.strategy_info)
-            if log.strategy == 'local':
-                if 'local_storage' not in fields:
-                    fields.append('local_storage')
-                log.local_storage = log.location
-            elif log.strategy == 'swift':
-                if 'swift_container' not in fields:
-                    fields.append('swift_container')
-                if 'swift_path' not in fields:
-                    fields.append('swift_path')
-                log.swift_container = strategy_info['container']
-                log.swift_path = log.location
-            else:
-                if 'location' not in fields:
-                    fields.append('location')
-
-        self._print_list(loglist, fields)
+        display_logs_for_single_workflow(self, app, revision)
 
 
 class OldAppCommands(cli_utils.CommandsBase):
@@ -1699,7 +1855,7 @@ Available commands:
     solum app list
         Print an index of all deployed applications.
 
-    solum app show <APP_NAME|UUID>
+    solum app show <NAME|UUID>
         Print detailed information about one application.
 
     solum app create [--app-file <AppFile>] [--git-url <GIT_URL>]
@@ -1714,14 +1870,22 @@ Available commands:
 
         Register a new application with Solum.
 
-    solum app deploy <APP_NAME|UUID>
+    solum app deploy <NAME|UUID>
         Deploy an application, building any applicable artifacts first.
+        du-id is optional flag. It can be used to pass in id of a previously
+        created deployment unit. If passed, this command will deploy the du
+        referenced by the provided du-id instead of building one first.
 
-    solum app delete <APP_NAME|UUID>
+    solum app delete <NAME|UUID>
         Delete an application and all related artifacts.
 
-    solum app logs <APP_NAME|UUID>
-        Show the logs of an application for all the deployments.
+    solum app logs <NAME|UUID> [--wf-id <wf-id>]
+        Show the logs of an application for all the workflows.
+        wf-id is optional flag which can be used to pass in id of one of
+        the existing workflows. If provided, the logs only for that workflow
+        are displayed.
+
+    solum app scale <APP_NAME|UUID> <target>
 
     solum workflow list <APP_NAME|UUID>
         List all application workflows.
